import asyncio
from datetime import datetime
import httpx

async def get(id_: int, url: str) -> str:
    st = datetime.now()
    async with httpx.AsyncClient() as ac:
        print(f"{id_} started")
        # この行に到達して非同期I/Oを使った通信が開始されると
        # そのコルーチンは中断され、ほかの実行可能なコルーチンが動き出す
        res = await ac.get(url)
        # レスポンスが返ってきて再開可能になったコルーチンから順次動き出す
        print(
            f"{id_} finished: {datetime.now() - st}"
        )
    return f"{id_}:{res.status_code}"

async def main():
    print(f"main started")
    st = datetime.now()
    result = await asyncio.gather(
        get(1, "https://api.github.com/orgs/python"),
        get(2, "https://api.github.com/orgs/pydantic"),
        get(3, "https://api.github.com/orgs/encode"),
    )
    print(f"{result=}")
    print(f"main finished: {datetime.now() - st}")

if __name__ == "__main__":
    asyncio.run(main())
